# coding:utf-8
import subprocess
import copy

from xml.etree import ElementTree


STATUS_RUNNING = "Running"


class FarmRoleException(Exception):
    pass

class FarmRoleNotFound(FarmRoleException):
    pass

class DuplicateFarmRole(FarmRoleException):
    pass


class Server(object):
    def __init__(self, index, location, internal_ip, external_ip, status):
        self.index = index
        self.location = location
        self.internal_ip = internal_ip
        self.external_ip = external_ip
        self.status = status

    @classmethod
    def from_xml(cls, xml):
        attrs = xml.attrib
        return Server(int(attrs["index"]), attrs["cloud-location"], attrs["internal-ip"], attrs["external-ip"],
                      attrs["status"])


class FarmRole(object):
    def __init__(self, id, servers):
        self.id = id
        self.servers = servers

    @classmethod
    def from_xml(cls, xml):
        id = int(xml.attrib["id"])
        servers = []
        for server_xml in xml.iterfind("./hosts/host"):
            servers.append(Server.from_xml(server_xml))
        return FarmRole(id, servers)

    @property
    def running_servers(self):
        for server in self.servers:
            if server.status == STATUS_RUNNING:
                yield server

    def __str__(self):
        return "<FarmRole: {0}>".format(self.id)


class FarmRoleEngine(object):
    def _szradm(self, params):
        """
        Make a call to szradm, check for errors, and return the output
        """

        params = copy.copy(params)
        params.insert(0, "/usr/local/bin/szradm")
        proc = subprocess.Popen(params, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if proc.wait():
            raise FarmRoleException("Unable to access szradm: %s", proc.stderr.read())
        return ElementTree.parse(proc.stdout)

    def farm_role_by_id(self, farm_role_id):
        args = ["-q", "list-roles"]
        tree = self._szradm(args)

        farm_role_xml = tree.find("./roles/role[@id='{0}']".format(farm_role_id))

        if farm_role_xml is None:
            raise FarmRoleNotFound(farm_role_id)

        return FarmRole.from_xml(farm_role_xml)

    def current_farm_role_id(self):
        args = ["-q", "get-server-user-data"]
        tree = self._szradm(args)

        node = tree.find("./user-data/key[@name='farm_roleid']/value")
        id = int(node.text.strip())

        return id





# coding:utf-8
import os

from casslr.lib.szradm import FarmRoleNotFound
from casslr.test.util import FarmRoleEngineTestCase


class FindRoleTestCase(FarmRoleEngineTestCase):
    def setUp(self):
        super(FindRoleTestCase, self).setUp()
        with open(os.path.join(self.test_data, "roles-valid.xml")) as f:
            self.engine.responses = [f.read()]

    def test_find_farm_role(self):
        id = 67453

        role = self.engine.farm_role_by_id(id)

        self.assertEqual(id, role.id)
        self.assertEqual(2, len(role.servers))

        s0, s1 = role.servers

        self.assertEqual(1, s0.index)
        self.assertEqual(2, s1.index)

        self.assertEqual("54.81.242.68", s0.external_ip)
        self.assertEqual("10.184.45.97", s1.internal_ip)

    def test_find_farm_role_other(self):
        self.assertEqual(1, len(self.engine.farm_role_by_id(67455).servers))

    def test_no_farm_role(self):
        self.assertRaises(FarmRoleNotFound, self.engine.farm_role_by_id, 99999)


class UserDataTestCase(FarmRoleEngineTestCase):
    def setUp(self):
        super(UserDataTestCase, self).setUp()
        with open(os.path.join(self.test_data, "user-data.xml")) as f:
            self.engine.responses = [f.read()]

    def test_farm_role_id(self):
        self.assertEqual(123, self.engine.current_farm_role_id())



# coding:utf-8
import os

from casslr.app import app

from casslr.test.util import FarmRoleEngineTestCase


class CasslrTestCase(FarmRoleEngineTestCase):
    def setUp(self):
        super(CasslrTestCase, self).setUp()

        for response in ["user-data.xml", "roles-valid.xml"]:
            with open(os.path.join(self.test_data, response)) as f:
                self.engine.responses.append(f.read())

        app.config["ENGINE"] = self.engine
        self.client = app.test_client()

    def test_seeds(self):
        response = self.client.get('/seeds')
        self.assertEqual(200, response.status_code)

        seeds = response.data.decode("utf-8").split(",")
        self.assertEqual(3, len(seeds))

        self.assertEqual(set(("10.190.214.199", "10.157.42.56", "10.160.20.42")), set(seeds))



# coding:utf-8
from flask import Flask

from casslr.lib.szradm import FarmRoleEngine

DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = 8020

app = Flask(__name__)
app.config.update({
    "ENGINE": FarmRoleEngine,
})


@app.route('/seeds')
def hello_world():
    engine = app.config["ENGINE"]
    farm_role_id = engine.current_farm_role_id()
    farm_role = engine.farm_role_by_id(farm_role_id)

    seeds_by_location = {}
    for server in farm_role.running_servers:
        if server.location not in seeds_by_location:
            seeds_by_location[server.location] = server.internal_ip
    return ",".join(seeds_by_location.values())


if __name__ == '__main__':
    app.run(host=DEFAULT_HOST, port=DEFAULT_PORT)




# coding:utf-8
import os
import unittest
from xml.etree import ElementTree
from casslr.lib import szradm


class TestFarmRoleEngine(szradm.FarmRoleEngine):
    def __init__(self):
        self.responses = []
        self.params = []

    def _szradm(self, params):
        self.params.append(params)
        return ElementTree.fromstring(self.responses.pop(0))


class FarmRoleEngineTestCase(unittest.TestCase):
    def setUp(self):
        self.test_data = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
        self.engine = TestFarmRoleEngine()


#coding:utf-8
import setuptools

name = "casslr"

with open("{0}/version.py".format(name)) as f:
    code = compile(f.read(), "version.py", "exec")
    exec(code)


setuptools.setup(
    name=name,
    version=__version__,
    packages=setuptools.find_packages(),
    url="https://github.com/scalr-tutorials/casslr",
    license="Apache 2.0",
    author="Thomas Orozco",
    author_email="thomas@scalr.com",
    long_description=open("README.rst").read(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: Apache Software License",
    ],
    entry_points={},
    install_requires=["flask"],
    extras_require={},
    setup_requires=["nose"],
    tests_require=["tox", "nose"],
)
